from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import cv2
import torch
import vot2022
import sys
import time
import os
import numpy as np
from vot2022 import Rectangle
from lib.test.tracker.mixformerrgbd_online import MixFormerRGBDOnline
import lib.test.parameter.mixformerd_online as vot_params

# filepath = os.path.abspath(__file__)
# AR_dir = os.path.abspath(os.path.join(os.path.dirname(filepath), "../../"))
# sys.path.append(AR_dir)
def get_transformed_img(rgb_path, depth_path, dtype='rgbcolormap'):
    if rgb_path:
        rgb = cv2.imread(rgb_path)
        rgb = cv2.cvtColor(rgb, cv2.COLOR_BGR2RGB)
    else:
        rgb = None

    if depth_path:
        dp = cv2.imread(depth_path, -1)
        max_depth = min(np.median(dp) * 3, 10000)
        dp[dp>max_depth] = max_depth
    else:
        dp = None

    if dtype == 'colormap':
        dp = cv2.normalize(dp, None, alpha=0, beta=255, norm_type=cv2.NORM_MINMAX)
        dp = np.asarray(dp, dtype=np.uint8)
        img = cv2.applyColorMap(dp, cv2.COLORMAP_JET)
    elif dtype == '3xD':
        dp = cv2.normalize(dp, None, alpha=0, beta=255, norm_type=cv2.NORM_MINMAX)
        dp = np.asarray(dp, dtype=np.uint8)
        img = cv2.merge((dp, dp, dp))
    if dtype == 'colormap3xD':
        dp1 = cv2.normalize(dp, None, alpha=0, beta=255, norm_type=cv2.NORM_MINMAX)
        dp1 = np.asarray(dp1, dtype=np.uint8)
        colormap = cv2.applyColorMap(dp1, cv2.COLORMAP_JET)
        dp = cv2.normalize(dp, None, alpha=0, beta=255, norm_type=cv2.NORM_MINMAX)
        dp = np.asarray(dp, dtype=np.uint8)
        depth = cv2.merge((dp, dp, dp))
        img = cv2.merge((colormap, depth))
    else:
        print('No such dtype !!! ')
        img = None

    return img


class MIXFORMERRGB_D(object):
    def __init__(self, tracker):
        self.tracker = tracker

    def initialize(self, img_d, region):
        self.H, self.W, _ = img_d.shape
        gt_bbox_np = np.array(region).astype(np.float32)
        '''Initialize STARK for specific video'''
        init_info = {'init_bbox': list(gt_bbox_np)}  # input must be (x,y,w,h)
        self.tracker.initialize(img_d, init_info)

    def track(self, img_d):
        '''TRACK'''
        '''base tracker'''
        outputs = self.tracker.track(img_d)
        pred_bbox = outputs['target_bbox']
        pred_score = outputs['target_score']
        return pred_bbox, pred_score

params = vot_params.parameters("large_finetune4", model="MixFormerRGBDOnlineScore_ep0015.pth.tar")
depth_type = params.cfg.DATA.DEPTH_TYPE
mixformer = MixFormerRGBDOnline(params, "VOT2022D")
tracker = MIXFORMERRGB_D(tracker=mixformer)
handle = vot2022.VOT("rectangle", "depth")
selection = handle.region()
imagefile = handle.frame()
init_box = [selection.x, selection.y, selection.width, selection.height]
if not imagefile:
    sys.exit(0)
imagefile_d = imagefile
# print(f"imagefile: {imagefile}")
image = get_transformed_img(None, imagefile_d, dtype='colormap3xD')  # Right

tracker.H = image.shape[0]
tracker.W = image.shape[1]

tracker.initialize(image, init_box)

while True:
    imagefile = handle.frame()
    if not imagefile:
        break
    imagefile_d = imagefile
    image = get_transformed_img(None, imagefile_d, dtype='colormap3xD')  # Right
    region, confidence = tracker.track(image)
    selection = Rectangle(region[0], region[1], region[2], region[3])
    handle.report(selection, confidence)